@@ -1,5 +1,4 @@
-# -*- coding: utf-8 -*-
-"""
+r"""
 Display ultimately customizable weather.
 
 This module allows you to specify an icon for nearly every weather scenario
@@ -44,8 +43,6 @@ Configuration parameters:
         (default 3)
     forecast_include_today: Include today in the forecast? (Boolean)
         (default False)
-    forecast_text_separator: Separator between entries in the forecast
-        (default ' ')
     format: How to display the weather
         This also dictates the type of forecast. The placeholders here refer to
         the format_[...] variables found below.
@@ -64,6 +61,8 @@ Configuration parameters:
         for future weather. Notably, this does not include information about
         sunrise or sunset times.
         (default '{icon}')
+    format_forecast_separator: Separator between entries in the forecast
+        (default ' ')
     format_humidity: Formatting for humidity (percentage)
         Available placeholders:
             icon, humidity
@@ -287,10 +286,10 @@ OWM_WEATHER_ICON = "//weather:0/id"
 OWM_WIND = "//wind"
 
 # Units constants
-RAIN_UNITS = set(["cm", "ft", "in", "mm", "m", "yd"])
+RAIN_UNITS = {"cm", "ft", "in", "mm", "m", "yd"}
 SNOW_UNITS = RAIN_UNITS
-TEMP_UNITS = set(["c", "f", "k"])
-WIND_UNITS = set(["fsec", "msec", "mph", "kmh"])
+TEMP_UNITS = {"c", "f", "k"}
+WIND_UNITS = {"fsec", "msec", "mph", "kmh"}
 
 # Conversion factors
 FT_FROM_METER = 3.28084
@@ -300,7 +299,7 @@ MPH_FROM_MSEC = 2.23694
 
 # Thresholds options
 THRESHOLDS_ALL = "all"
-THRESHOLDS_NAMES = set([THRESHOLDS_ALL, "current", "min", "max"])
+THRESHOLDS_NAMES = {THRESHOLDS_ALL, "current", "min", "max"}
 
 # Thresholds defaults
 THRESHOLDS = {"all": [(-100, "#0FF"), (0, "#00F"), (50, "#0F0"), (150, "#FF0")]}
@@ -317,31 +316,31 @@ class Py3status:
     country = None
     forecast_days = 3
     forecast_include_today = False
-    forecast_text_separator = " "
     format = "{city} {icon} {temperature}[ {rain}], {description} {forecast}"
     format_clouds = "{icon} {coverage}%"
     format_forecast = "{icon}"
+    format_forecast_separator = " "
     format_humidity = "{icon} {humidity}%"
     format_pressure = "{icon} {pressure} hPa"
-    format_rain = "[\?if=amount {icon} {amount:.0f} {unit}]"
-    format_snow = "[\?if=amount {icon} {amount:.0f} {unit}]"
+    format_rain = r"[\?if=amount {icon} {amount:.0f} {unit}]"
+    format_snow = r"[\?if=amount {icon} {amount:.0f} {unit}]"
     format_sunrise = "{icon} %-I:%M %p"
     format_sunset = "{icon} %-I:%M %p"
-    format_temperature = u"{icon} [\?color=all {current:.0f}°{unit}]"
-    format_wind = "[\?if=speed {icon} {speed:.0f} {unit}]"
-    icon_atmosphere = u"🌫"
-    icon_cloud = u"☁"
-    icon_extreme = u"⚠"
-    icon_humidity = u"●"
-    icon_pressure = u"◌"
-    icon_rain = u"🌧"
-    icon_snow = u"❄"
-    icon_sun = u"☼"
-    icon_sunrise = u"⇑"
-    icon_sunset = u"⇓"
-    icon_temperature = u"○"
-    icon_thunderstorm = u"⛈"
-    icon_wind = u"☴"
+    format_temperature = r"{icon} [\?color=all {current:.0f}°{unit}]"
+    format_wind = r"[\?if=speed {icon} {speed:.0f} {unit}]"
+    icon_atmosphere = "🌫"
+    icon_cloud = "☁"
+    icon_extreme = "⚠"
+    icon_humidity = "●"
+    icon_pressure = "◌"
+    icon_rain = "🌧"
+    icon_snow = "❄"
+    icon_sun = "☼"
+    icon_sunrise = "⇑"
+    icon_sunset = "⇓"
+    icon_temperature = "○"
+    icon_thunderstorm = "⛈"
+    icon_wind = "☴"
     icons = None
     lang = "en"
     location = None
@@ -352,7 +351,16 @@ class Py3status:
     unit_wind = "mph"
 
     class Meta:
-        deprecated = {"remove": [{"param": "offset_gmt", "msg": "obsolete"}]}
+        deprecated = {
+            "remove": [{"param": "offset_gmt", "msg": "obsolete"}],
+            "rename": [
+                {
+                    "param": "forecast_text_separator",
+                    "new": "format_forecast_separator",
+                    "msg": "obsolete parameter, use format_forecast_separator",
+                }
+            ],
+        }
 
     def _get_icons(self):
         if self.icons is None:
@@ -377,15 +385,15 @@ class Py3status:
         for source in (defaults, self.icons):
             for key in source:
                 if not key.replace("_", "").isdigit():
-                    raise Exception("Invalid icon id: (%s)" % key)
+                    raise Exception(f"Invalid icon id: ({key})")
 
                 if "_" in key:
                     if key.count("_") != 1:
-                        raise Exception("Invalid icon range: %s" % key)
+                        raise Exception("fInvalid icon range: {key}")
 
                     # Populate each code
-                    (start, end) = tuple(map(int, key.split("_")))
-                    for code in range(start, end + 1):
+                    start, _, end = key.partition("_")
+                    for code in range(int(start), int(end) + 1):
                         data[code] = source[key]
 
                 else:
@@ -416,15 +424,22 @@ class Py3status:
             raise Exception("unit_wind is not recognized")
 
         # Check thresholds for validity
-        if set(self.thresholds.keys()) > THRESHOLDS_NAMES:
+        if set(self.thresholds) > THRESHOLDS_NAMES:
             raise Exception("threshold name(s) are not recognized")
 
         # Copy thresholds if available
         if THRESHOLDS_ALL in self.thresholds:
-            for name in THRESHOLDS_NAMES - set([THRESHOLDS_ALL]):
+            for name in THRESHOLDS_NAMES - {THRESHOLDS_ALL}:
                 if name not in self.thresholds:
                     self.thresholds[name] = self.thresholds[THRESHOLDS_ALL]
 
+        # Initialize per-format thresholds
+        self.thresholds_init = {}
+        for name in ("format_humidity",):
+            self.thresholds_init[name] = self.py3.get_color_names_list(
+                getattr(self, name)
+            )
+
     def _make_req(self, url, params=None):
         # Make a request expecting a JSON response
         req = self.py3.request(url, params=params)
@@ -462,7 +477,7 @@ class Py3status:
 
     def _get_loc_tz_info(self):
         # Preference a user-set location
-        if all(map(lambda x: x is not None, (self.location, self.city, self.country))):
+        if all(x is not None for x in (self.location, self.city, self.country)):
             return (self.location, self.city, self.country)
 
         data = {}
@@ -516,7 +531,7 @@ class Py3status:
         data = self._make_req(OWM_FUTURE_ENDPOINT, extras)
         # Extract forecast
         weathers = data["list"]
-        return weathers[:-1] if (self.forecast_include_today) else weathers[1:]
+        return weathers[:-1] if self.forecast_include_today else weathers[1:]
 
     def _get_icon(self, wthr):
         # Lookup the icon from the weather code (default sunny)
@@ -618,11 +633,16 @@ class Py3status:
 
     def _format_humidity(self, wthr):
         # Format the humidity (default zero humidity)
-        humidity = self._jpath(wthr, OWM_HUMIDITY, 0)
+        humidity_data = {
+            "icon": self.icon_humidity,
+            "humidity": self._jpath(wthr, OWM_HUMIDITY, 0),
+        }
 
-        return self.py3.safe_format(
-            self.format_humidity, {"icon": self.icon_humidity, "humidity": humidity}
-        )
+        for x in self.thresholds_init["format_humidity"]:
+            if x in humidity_data:
+                self.py3.threshold_get_color(humidity_data[x], x)
+
+        return self.py3.safe_format(self.format_humidity, humidity_data)
 
     def _format_pressure(self, wthr):
         # Get data and add the icon
@@ -641,7 +661,7 @@ class Py3status:
             return val - 273.15
 
         def kToF(val):
-            return val * (9.0 / 5.0) - 459.67
+            return val * 9 / 5 - 459.67
 
         options = {
             "c": {
@@ -667,7 +687,7 @@ class Py3status:
         choice["unit"] = self.unit_temperature
 
         # Calculate thresholds
-        for name in THRESHOLDS_NAMES - set([THRESHOLDS_ALL]):
+        for name in THRESHOLDS_NAMES - {THRESHOLDS_ALL}:
             # Try to apply the specific threshold
             if name in self.thresholds:
                 self.py3.threshold_get_color(choice[name], name)
@@ -725,8 +745,9 @@ class Py3status:
             forecasts.append(self.py3.safe_format(self.format_forecast, future))
 
         # Give the final format
+        format_forecast_separator = self.py3.safe_format(self.format_forecast_separator)
         today["forecast"] = self.py3.composite_join(
-            self.forecast_text_separator, forecasts
+            format_forecast_separator, forecasts
         )
 
         return self.py3.safe_format(self.format, today)
@@ -795,9 +816,9 @@ if __name__ == "__main__":
             "format_rain": "{icon} {amount:.0f} in",
             "format_snow": "{icon} {amount:.0f} in",
             "format_temperature": (
-                "{icon}: max: [\?color=max {max:.0f}°F], "
-                "min: [\?color=min {min:.0f}°F], "
-                "current: [\?color=current {current:.0f}°F]"
+                r"{icon}: max: [\?color=max {max:.0f}°F], "
+                r"min: [\?color=min {min:.0f}°F], "
+                r"current: [\?color=current {current:.0f}°F]"
             ),
             "format_wind": (
                 "{icon} {degree}°, gust: {gust:.0f} mph, " "speed: {speed:.0f} mph"
@@ -806,6 +827,6 @@ if __name__ == "__main__":
             "format_forecast": ("{icon} " + all_string),
             # Miscellaneous
             "forecast_days": 1,
-            "forecast_text_separator": "//",
+            "format_forecast_separator": "//",
         },
     )
